/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** Service Class Executive (SCE)
** - Move Storage Class Provider
**
** Author: 	Andrew Hewett
**		Universitaet Oldenburg, Germany
** Created:	08/1993
**
** Last Update:		$Author: hewett $
** Update Date:		$Date: 1996/09/27 08:46:22 $
** Source File:		$Source: /share/dicom/cvs-depot/dcmtk/imagectn/apps/scemove.cc,v $
** CVS/RCS Revision:	$Revision: 1.5 $
** Status:		$State: Exp $
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <errno.h>
END_EXTERN_C

#include "dicom.h"
#include "imagectn.h"
#include "dimse.h"
#include "diutil.h"
#include "sce.h"
#include "imagedb.h"
#include "cnf.h"
#ifndef NODISPLAY
#include "dispuser.h"
#endif
#include "dcdeftag.h"

#include "scemove.h"

BOOLEAN respondIntermediatePending = TRUE;
BOOLEAN strictMoveRequirements = FALSE;

/*
 * C-MOVE SCP
 */


typedef struct {
    DB_Handle	*dbHandle;
    DIC_US	priorStatus;

    T_ASC_Association   *origAssoc;	/* association of requestor */
    T_ASC_Association   *subAssoc;	/* sub-association */

    BOOLEAN assocStarted;	/* true if the association was started */
    BOOLEAN allowImageDisplay;	/* true if 1st image should be displayed */
    
    DIC_US origMsgId;		/* message id of request */
    DIC_AE origAETitle;		/* title of requestor */
    DIC_NODENAME origHostName;	/* hostname of move requestor */

    T_DIMSE_Priority priority;	/* priority of move request */
    DIC_AE ourAETitle;  	/* our current title */
    DIC_AE dstAETitle;		/* destination title for move */

    char *failedUIDs;		/* instance UIDs of failed store sub-ops */

    DIC_US nRemaining; 
    DIC_US nCompleted; 
    DIC_US nFailed; 
    DIC_US nWarning;

} SCE_MoveContext;

static void
addFailedUIDInstance(SCE_MoveContext *context, const char *sopInstance)
{
    int len;

    if (context->failedUIDs == NULL) {
	if ((context->failedUIDs = (char*)malloc(DIC_UI_LEN+1)) == NULL) {
	    errmsg("malloc failure: addFailedUIDInstance");
	    return;
	}
	strcpy(context->failedUIDs, sopInstance);
    } else {
	len = strlen(context->failedUIDs);
	if ((context->failedUIDs = (char*)realloc(context->failedUIDs, 
	    (len+strlen(sopInstance)+2))) == NULL) {
	    errmsg("realloc failure: addFailedUIDInstance");
	    return;
	}
	/* tag sopInstance onto end of old with '\' between */
	strcat(context->failedUIDs, "\\");
	strcat(context->failedUIDs, sopInstance);
    }
}

static void
moveSubOpProgressCallback(void * /*callbackData*/, 
    T_DIMSE_StoreProgress *progress,
    T_DIMSE_C_StoreRQ * /*req*/)
{
    int percent;
    static int lastPercent = 0;

    switch (progress->state) {
    case DIMSE_StoreBegin:
	if (verbose) printf("XMIT:");
	lastPercent = 0;
	break;
    case DIMSE_StoreEnd:
	if (verbose) printf("\n");
	if (displayTransferProgress) {
#ifndef NODISPLAY
	    DSU_operationState(DSU_SubAssociation, DSU_StoreOperation,
		DSU_RunningOperation, DSU_Outgoing, 100);
#endif
	}
	break;
    default:
	if (verbose) putchar('.');
	if (progress->totalBytes == 0) {
	    percent = 100;
	} else {
	    percent = (int)(((float)progress->progressBytes / 
	        (float)progress->totalBytes) * 100.0);
	}
	if (displayTransferProgress) {
	    if ((percent - lastPercent) >= 10) {	/* 10% change */
		if (lastPercent < 100) {
#ifndef NODISPLAY
	            DSU_operationState(DSU_SubAssociation, DSU_StoreOperation,
		        DSU_RunningOperation, DSU_Outgoing, percent);
#endif
		}
	        lastPercent = percent;
	    }
	}
	break;
    }
    if (verbose) fflush(stdout);
}

static CONDITION
performMoveSubOp(SCE_MoveContext *context, 
	DIC_UI sopClass, DIC_UI sopInstance,
	char *fname)
{
    CONDITION cond = DIMSE_NORMAL;
    T_DIMSE_C_StoreRQ req;
    T_DIMSE_C_StoreRSP rsp;
    DIC_US msgId;
    T_ASC_PresentationContextID presId;
    DcmDataset *statusDetail = NULL;
    int lockfd;

    /* shared lock image file */
    lockfd = open(fname, O_RDONLY, 0666);
    if (lockfd < 0) {
        /* due to quota system the file could have been deleted */
	errmsg("Move SCP: storeSCU: [file: %s]: %s", 
	    fname, strerror(errno));
	context->nFailed++;
	addFailedUIDInstance(context, sopInstance);
	return DIMSE_NORMAL;
    }
    flock(lockfd, LOCK_SH);

    msgId = context->subAssoc->nextMsgID++;
 
    /* which presentation context should be used */
    presId = ASC_findAcceptedPresentationContextID(context->subAssoc,
        sopClass);
    if (presId == 0) {
	context->nFailed++;
	addFailedUIDInstance(context, sopInstance);
	errmsg("Move SCP: storeSCU: [file: %s] No presentation context for: (%s) %s", 
	    fname, DU_sopClassToModality(sopClass), sopClass);
	return COND_PushCondition(DIMSE_NOVALIDPRESENTATIONCONTEXTID, 
	    "DIMSE: No Presentation Context for: %s", sopClass);
    }

    req.MessageID = msgId;
    strcpy(req.AffectedSOPClassUID, sopClass);
    strcpy(req.AffectedSOPInstanceUID, sopInstance);
    req.DataSetType = DIMSE_DATASET_PRESENT;
    req.Priority = context->priority;
    req.opts = (O_STORE_MOVEORIGINATORAETITLE | O_STORE_MOVEORIGINATORID);
    strcpy(req.MoveOriginatorApplicationEntityTitle,
	context->origAETitle);
    req.MoveOriginatorID = context->origMsgId;

    if (verbose) {
	printf("Store SCU RQ: MsgID %d, (%s)\n", 
	    msgId, DU_sopClassToModality(sopClass));
    }

    cond = DIMSE_storeUser(context->subAssoc, presId, &req,
        fname, NULL, moveSubOpProgressCallback, NULL, 
	DIMSE_BLOCKING, 0, 
	&rsp, &statusDetail);
	
    /* unlock image file */
    flock(lockfd, LOCK_UN);
    close(lockfd);
	
    if (cond == DIMSE_NORMAL) {
        if (verbose) {
	    printf("Move SCP: Received Store SCU RSP [Status=%s]\n",
	        DU_cstoreStatusString(rsp.Status));
        }
	if (rsp.Status == STATUS_Success) {
	    /* everything ok */
	    context->nCompleted++;
	} else if ((rsp.Status & 0xf000) == 0xb000) {
	    /* a warning status message */
	    context->nWarning++;
	    errmsg("Move SCP: Store Waring: Response Status: %s", 
		DU_cstoreStatusString(rsp.Status));
	} else {
	    context->nFailed++;
	    addFailedUIDInstance(context, sopInstance);
	    /* print a status message */
	    errmsg("Move SCP: Store Failed: Response Status: %s", 
		DU_cstoreStatusString(rsp.Status));
	}
    } else {
	context->nFailed++;
	addFailedUIDInstance(context, sopInstance);
	errmsg("Move SCP: storeSCU: Store Request Failed:");
	COND_DumpConditions();
    }
    if (statusDetail != NULL) {
        if (verbose) {
	    printf("  Status Detail:\n");
	    statusDetail->print();
	}
        delete statusDetail;
    }
    return cond;
}

static BOOLEAN
mapMoveDestination(const char *origPeer, const char *origAE,
	const char *dstAE, char *dstPeer, int *dstPort)
{
    /*
     * This routine enforces RSNA'93 Demo Requirements regarding
     * the destination of move commands.
     * 
     */
    BOOLEAN ok = FALSE;
    const char *dstPeerName; /* the CNF utility returns us a static char* */

    if (restrictMoveToSameAE) {
        /* AE Titles the same ? */
        ok = (strcmp(origAE, dstAE) == 0);
        if (!ok) {
	    if (verbose) {
	        printf("mapMoveDestination: strictMove Reqs: '%s' != '%s'\n",
		    origAE, dstAE); 
	    }
	    return FALSE;
	}
    }
    
    ok = CNF_peerForAETitle((char*)dstAE, &dstPeerName, dstPort);
    if (!ok) {
        if (verbose) {
	    printf("mapMoveDestination: unknown AE: '%s'\n", dstAE);
	}
        return FALSE;	/* dstAE not known */
    }
    
    strcpy(dstPeer, dstPeerName);

    if (restrictMoveToSameHost) {
	/* hosts the same ? */
	ok = (strcmp(origPeer, dstPeer) == 0);
	if (!ok) {
	    if (verbose) {
		printf("mapMoveDestination: different hosts: '%s', '%s'\n",
		       origPeer, dstPeer);
	    }
	    return FALSE;
	}
    }

    if (restrictMoveToSameVendor) {
	/* AE titles belong to the same vendor */
	ok = CNF_checkForSameVendor((char*)origAE, (char*)dstAE);
	if (!ok) {
	    if (verbose) {
		printf("mapMoveDestination: different vendors: '%s', '%s'\n",
		       origAE, dstAE);
	    }
	    return FALSE;
	}
    }
    
    return ok;
}

static CONDITION
addAllStoragePresentationContexts(T_ASC_Parameters *params)
{
    CONDITION cond = ASC_NORMAL;

    int i;
    int pid = 1;

    /* 
    ** We prefer to accept Explicitly encoded transfer syntaxes.
    ** If we are running on a Little Endian machine we prefer 
    ** LittleEndianExplicitTransferSyntax to BigEndianTransferSyntax.
    ** Some SCP implementations will just select the first transfer
    ** syntax they support (this is not part of the standard) so
    ** organise the proposed transfer syntaxes to take advantage
    ** of such behaviour.
    */

    const char* transferSyntaxes[] = { 
	NULL, NULL, UID_LittleEndianImplicitTransferSyntax };

    /* gLocalByteOrder is defined in dcxfer.h */
    if (gLocalByteOrder == EBO_LittleEndian) {
	/* we are on a little endian machine */
	transferSyntaxes[0] = UID_LittleEndianExplicitTransferSyntax;
	transferSyntaxes[1] = UID_BigEndianExplicitTransferSyntax;
    } else {
	/* we are on a big endian machine */
	transferSyntaxes[0] = UID_BigEndianExplicitTransferSyntax;
	transferSyntaxes[1] = UID_LittleEndianExplicitTransferSyntax;
    }

    for (i=0; i<numberOfDcmStorageSOPClassUIDs && SUCCESS(cond); i++) {
	cond = ASC_addPresentationContext(
	    params, pid, dcmStorageSOPClassUIDs[i],
	    transferSyntaxes, DIM_OF(transferSyntaxes));
	pid += 2;	/* only odd presentation context id's */
    }
    return cond;
}

static CONDITION
buildSubAssociation(T_DIMSE_C_MoveRQ *request, SCE_MoveContext *context)
{
    CONDITION cond = ASC_NORMAL;
    DIC_NODENAME dstHostName;
    DIC_NODENAME dstHostNamePlusPort;
    int dstPortNumber;
    DIC_NODENAME localHostName;
    T_ASC_Parameters *params;

    strcpy(context->dstAETitle, request->MoveDestination);
    /*
     * We must map the destination AE Title into a host name and port
     * address.  Further, we must make sure that the RSNA'93 demonstration
     * rules are observed regarding move destinations. 
     */
    ASC_getAPTitles(context->origAssoc->params, context->origAETitle,
	context->ourAETitle, NULL);
    ASC_getPresentationAddresses(context->origAssoc->params, 
        context->origHostName, NULL);

    if (!mapMoveDestination(context->origHostName, context->origAETitle,
	request->MoveDestination, dstHostName, &dstPortNumber)) {
	return APP_INVALIDPEER;
    }
    if (SUCCESS(cond)) {
	cond = ASC_createAssociationParameters(&params, ASC_DEFAULTMAXPDU);
	if (!SUCCESS(cond)) {
	    errmsg("moveSCP: Cannot create Association-params for sub-ops:");
	    COND_DumpConditions();
	}
    }
    if (SUCCESS(cond)) {
	gethostname(localHostName, sizeof(localHostName) - 1);
	sprintf(dstHostNamePlusPort, "%s:%d", dstHostName, dstPortNumber);
	ASC_setPresentationAddresses(params, localHostName, 
		dstHostNamePlusPort);
	ASC_setAPTitles(params, context->ourAETitle, context->dstAETitle,NULL);

	cond = addAllStoragePresentationContexts(params);
	if (!SUCCESS(cond)) {
	    COND_DumpConditions();
	}
	if (debug) {
	    printf("Request Parameters:\n");
	    ASC_dumpParameters(params);
	}
    }
    if (SUCCESS(cond)) {
	/* create association */
	if (verbose)
	    printf("Requesting Sub-Association\n");
	cond = ASC_requestAssociation(net, params,
				      &context->subAssoc);
	if (cond != ASC_NORMAL) {
	    if (cond == ASC_ASSOCIATIONREJECTED) {
		T_ASC_RejectParameters rej;

		ASC_getRejectParameters(params, &rej);
		errmsg("moveSCP: Sub-Association Rejected");
		ASC_printRejectParameters(stderr, &rej);
		fprintf(stderr, "\n");
		COND_PopCondition(FALSE); /* pop only the reject condition */
	    } else {
		errmsg("moveSCP: Sub-Association Request Failed:");
		COND_DumpConditions();
		COND_PopCondition(TRUE);
	    }
	}
    }

    if (cond == ASC_NORMAL) {
	context->assocStarted = TRUE;
	context->allowImageDisplay = displayImagesOnMove &&
	    peerCanDisplayImages(dstHostName);
        /* inform display that an association has started */
#ifndef NODISPLAY
        DSU_startAssociation(dstHostName, 
	    context->dstAETitle, context->ourAETitle,
	    DSU_Requestor, DSU_SubAssociation);
#endif
    }    
    return cond;
}

static CONDITION
closeSubAssociation(SCE_MoveContext *context)
{
    CONDITION cond = ASC_NORMAL;

    if (context->subAssoc != NULL) {
	/* release association */
	if (verbose)
	    printf("Releasing Sub-Association\n");
	cond = ASC_releaseAssociation(context->subAssoc);
	if (cond != ASC_NORMAL && cond != ASC_RELEASECONFIRMED) {
	    errmsg("moveSCP: Sub-Association Release Failed:");
	    COND_DumpConditions();
	}
	cond = ASC_dropAssociation(context->subAssoc);
	if (!SUCCESS(cond)) {
	    errmsg("moveSCP: Sub-Association Drop Failed:");
	    COND_DumpConditions();
	}
	cond = ASC_destroyAssociation(&context->subAssoc);
	if (!SUCCESS(cond)) {
	    errmsg("moveSCP: Sub-Association Destroy Failed:");
	    COND_DumpConditions();
	}

    }

    if (context->assocStarted) {
        /* inform display that association has ended */
#ifndef NODISPLAY
        DSU_stopAssociation(DSU_SubAssociation);
#endif
	context->assocStarted = FALSE;
    }

    return cond;
}

static void
moveNextImage(SCE_MoveContext * context, DB_Status * dbStatus)
{
    CONDITION cond = DIMSE_NORMAL;
    CONDITION dbcond = DB_NORMAL;
    DIC_UI subImgSOPClass;	/* sub-operation image SOP Class */
    DIC_UI subImgSOPInstance;	/* sub-operation image SOP Instance */
    char subImgFileName[MAXPATHLEN + 1];	/* sub-operation image file */
    static BOOLEAN firstImage = TRUE;
    DIC_AE peerAETitle;
    DIC_AE myAETitle;
    const char *vendorName;
    char displayString[256];

    /* clear out strings */
    bzero(subImgFileName, sizeof(subImgFileName));
    bzero(subImgSOPClass, sizeof(subImgSOPClass));
    bzero(subImgSOPInstance, sizeof(subImgSOPInstance));

    /* get DB response */
    dbcond = DB_nextMoveResponse(context->dbHandle,
	subImgSOPClass, subImgSOPInstance, subImgFileName,
	&context->nRemaining, dbStatus);
    if (dbcond != DB_NORMAL) {
	errmsg("moveSCP: Database: DB_nextMoveResponse Failed (%s):",
	    DU_cmoveStatusString(dbStatus->status));
    }

    if (dbStatus->status == STATUS_Pending) {
	/* inform display */
#ifndef NODISPLAY
	DSU_operationState(DSU_SubAssociation, DSU_StoreOperation,
	    DSU_StartOperation, DSU_Outgoing, 0);
#endif
	/* perform sub-op */
	cond = performMoveSubOp(context, subImgSOPClass,
	    subImgSOPInstance, subImgFileName);
	if (cond != DIMSE_NORMAL) {
	    errmsg("moveSCP: Move Sub-Op Failed:");
	    COND_DumpConditions();
	    COND_PopCondition(TRUE);	/* clear condition stack */
	}

	/* inform display */
#ifndef NODISPLAY
	DSU_operationState(DSU_SubAssociation, DSU_StoreOperation,
	    DSU_StopOperation, DSU_Outgoing, 100);
#endif
	if (context->allowImageDisplay && firstImage) {
            ASC_getAPTitles(context->subAssoc->params, myAETitle, 
	        peerAETitle, NULL);
            vendorName = CNF_vendorForPeerAETitle(peerAETitle);
	    sprintf(displayString, "%s <- %s", 
	        ((vendorName!=NULL)?(vendorName):(peerAETitle)),
	        myAETitle);
#ifndef NODISPLAY
	    DSU_displayIndependent(displayString, subImgFileName);
#endif
	    firstImage = FALSE;
	}
    }
}

static void
failAllSubOperations(SCE_MoveContext * context, DB_Status * dbStatus)
{
    CONDITION dbcond = DB_NORMAL;
    DIC_UI subImgSOPClass;	/* sub-operation image SOP Class */
    DIC_UI subImgSOPInstance;	/* sub-operation image SOP Instance */
    char subImgFileName[MAXPATHLEN + 1];	/* sub-operation image file */

    /* clear out strings */
    bzero(subImgFileName, sizeof(subImgFileName));
    bzero(subImgSOPClass, sizeof(subImgSOPClass));
    bzero(subImgSOPInstance, sizeof(subImgSOPInstance));

    while (dbStatus->status == STATUS_Pending) {
        /* get DB response */
        dbcond = DB_nextMoveResponse(context->dbHandle,
	    subImgSOPClass, subImgSOPInstance, subImgFileName,
	    &context->nRemaining, dbStatus);
        if (dbcond != DB_NORMAL) {
	    errmsg("moveSCP: Database: DB_nextMoveResponse Failed (%s):",
	        DU_cmoveStatusString(dbStatus->status));
        }

	if (dbStatus->status == STATUS_Pending) {
	    context->nFailed++;
	    addFailedUIDInstance(context, subImgSOPInstance);
	}
    }
    dbStatus->status = 
        STATUS_MOVE_Warning_SubOperationsCompleteOneOrMoreFailures;    
}

static void
buildFailedInstanceList(SCE_MoveContext * context,
    DcmDataset ** rspIds)
{
    BOOLEAN ok;

    if (context->failedUIDs != NULL) {
	*rspIds = new DcmDataset();
	ok = DU_putStringDOElement(*rspIds, DCM_FailedSOPInstanceUIDList,
	    context->failedUIDs);
	if (!ok) {
	    errmsg("moveSCP: failed to build DCM_FailedSOPInstanceUIDList");
	}
	free(context->failedUIDs);
	context->failedUIDs = NULL;
    }
}


static void 
moveCallback(
	/* in */ 
	void *callbackData,  
	BOOLEAN cancelled, T_DIMSE_C_MoveRQ *request, 
	DcmDataset *requestIdentifiers, int responseCount,
	/* out */
	T_DIMSE_C_MoveRSP *response, DcmDataset **statusDetail,	
	DcmDataset **responseIdentifiers)
{
    CONDITION cond;
    CONDITION dbcond;
    SCE_MoveContext *context;
    DB_Status dbStatus;

    context = (SCE_MoveContext*)callbackData;	/* recover context */

    dbStatus.status = context->priorStatus;
    dbStatus.statusDetail = NULL;
    
    if (responseCount == 1) {
        /* start the database search */
	if (verbose) {
	    printf("Move SCP Request Identifiers:\n");
	    requestIdentifiers->print();
        }
        dbcond = DB_startMoveRequest(context->dbHandle, 
	    request->AffectedSOPClassUID, requestIdentifiers, &dbStatus);
        if (dbcond != DB_NORMAL) {
	    errmsg("moveSCP: Database: DB_startMoveRequest Failed (%s):",
		DU_cmoveStatusString(dbStatus.status));
        }

        if (dbStatus.status == STATUS_Pending) {
            /* If we are going to be performing sub-operations, build
             * a new association to the move destination.
             */
	    cond = buildSubAssociation(request, context);
	    if (cond == APP_INVALIDPEER) {
	        dbStatus.status = STATUS_MOVE_Failed_MoveDestinationUnknown;
	    } else if (cond != ASC_NORMAL) {
	        /* failed to build association, must fail move */
		failAllSubOperations(context, &dbStatus);
	    }
        }
    }
    
    /* only cancel if we have pending status */
    if (cancelled && dbStatus.status == STATUS_Pending) {
	DB_cancelMoveRequest(context->dbHandle, &dbStatus);
    }

    if (dbStatus.status == STATUS_Pending) {
        moveNextImage(context, &dbStatus);
    }

    if (dbStatus.status != STATUS_Pending) {
	/*
	 * Tear down sub-association (if it exists).
	 */
	closeSubAssociation(context);

	/*
	 * Need to adjust the final status if any sub-operations failed or
	 * had warnings 
	 */
	if (context->nFailed > 0 || context->nWarning > 0) {
	    dbStatus.status = 
	        STATUS_MOVE_Warning_SubOperationsCompleteOneOrMoreFailures;
	}
    }
    
    if (verbose) {
        printf("Move SCP Response %d [status: %s]\n", responseCount,
	    DU_cmoveStatusString(dbStatus.status));
    }

    if (dbStatus.status != STATUS_Success && 
        dbStatus.status != STATUS_Pending) {
	/* 
	 * May only include response identifiers if not Success 
	 * and not Pending 
	 */
	buildFailedInstanceList(context, responseIdentifiers);
    }

    /* set response status */
    response->Status = dbStatus.status;
    response->NumberOfRemainingSubOperations = context->nRemaining;
    response->NumberOfCompletedSubOperations = context->nCompleted;
    response->NumberOfFailedSubOperations = context->nFailed;
    response->NumberOfWarningSubOperations = context->nWarning;
    *statusDetail = dbStatus.statusDetail;
    
}

CONDITION
SCE_moveSCP(T_ASC_Association * assoc, T_DIMSE_C_MoveRQ * request,
	T_ASC_PresentationContextID presID, DB_Handle *dbHandle)
{
    CONDITION cond;
    SCE_MoveContext context;

    bzero((char*)&context, sizeof(context));
    
    context.dbHandle = dbHandle;
    context.priorStatus = STATUS_Pending;
    context.origAssoc = assoc;
    context.origMsgId = request->MessageID;
    context.priority = request->Priority;
    ASC_getAPTitles(assoc->params, NULL, context.ourAETitle, NULL);
    context.assocStarted = FALSE;
    context.allowImageDisplay = TRUE;

    if (verbose) {
	printf("Received Move SCP: ");
	DIMSE_printCMoveRQ(stdout, request);
    }

    cond = DIMSE_moveProvider(assoc, presID, request, 
    	moveCallback, &context, DIMSE_BLOCKING, 0);
    if (!SUCCESS(cond)) {
        errmsg("Move SCP Failed:");
	COND_DumpConditions();
    }
    return cond; 
}


/*
** CVS Log
** $Log: scemove.cc,v $
** Revision 1.5  1996/09/27 08:46:22  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.4  1996/09/24 15:52:35  hewett
** Now uses global table of Storage SOP Class UIDs (from dcuid.h).
** Also added preliminary support for the Macintosh environment (GUSI library).
**
** Revision 1.3  1996/04/25 16:29:09  hewett
** Added char* parameter casts to bzero() calls.
**
** Revision 1.2  1996/04/22 10:25:38  hewett
** Implements optional restrictions on move destination.  Now uses a global
** DICOM network rather than a local instance.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
